# GameLiftServerSdk C++
## Documentation
You can find the official Amazon GameLift documentation [here](https://aws.amazon.com/documentation/gamelift/).
## Minimum requirements:
* Either of the following:  
  * Microsoft Visual Studio 2012 or later  
  * GNU Compiler Collection (GCC) 4.9 or later
* CMake version 3.1 or later
* A Git client available on the PATH.
* OpenSSL installation

## Pre-requisites

### OpenSSL

1. Install the full version of OpenSSL from https://wiki.openssl.org/index.php/Binaries for the appropriate platform
1. Follow the instcutions provided by OpenSSL for installation 
1. Once installed, navigate to the OpenSSL folder (ie C:\Program Files\OpenSSL-Win64) and **COPY** the following two DLLs and save them to a safe location for later (such as your desktop).  
***When uploading your application to Amazon GameLift, these two DLLs must be included with the server application.***
    * libssl-3-x64.dll
    * libcrypto-3-x64.dll
1. Finally, follow the below instrcutions for each specific platform
  
#### Mac

1. Add the following environment variables to your **.zshrc** file by running `vim ~/.zshrc`. Replace values in <> with the path to the OpenSSL directory.
    ```zsh
    export OPENSSL_ROOT_DIR=<PATH_TO_OPENSSL_DIR>
    export OPENSSL_LIBRARIES=<PATH_TO_OPENSSL_DIR>/lib
    ```
1. Once **.zshrc** is updated, you may need to run `source ~/.zshrc` to apply the change in each existing terminal sessions.

#### Windows

1. Note the directory where OpenSSL was installed
1. Open **System Properties > Environmental Varaibles**
1. Add your OpenSSL install directory to your PATH = `<PATH_TO_OPENSSL_DIR>\bin`
1. Create the following environment variables:
    ```
    OPENSSL_INCLUDE_DIR = <PATH_TO_OPENSSL_DIR>\include
    OPENSSL_LIBRARIES = <PATH_TO_OPENSSL_DIR>\lib
    OPENSSL_ROOT_DIR = <PATH_TO_OPENSSL_DIR>\OpenSSL
    ```

### CMake 3.1+

1. Install CMake 3.1+ from https://cmake.org/download/ for the appropriate platform
1. Follow instructions provided by CMake for installation

### Python 3.6+

1. Install Python 3.6+ from https://www.python.org/downloads/ for the appropriate platform
1. Follow instructions provided by the above site for installation

## Building the SDK
### Out of source build
To build the server sdk, all you must do is the following:  
#### Linux
```sh
mkdir out
cd out
cmake ..
make
```

#### Windows
1. Create the build directory 
    ```sh
    mkdir out
    cd out
    ```
1. Make the solution and project files  
*Note: you might see warnings such as `Detected a Windows Shared STD/NOSTD or Unreal build.  Skipping unit tests`, these are expected and just stating that for the specific options passed certain features are not included.*
    ```
    cmake -G "Visual Studio 15 2017 Win64" ..
    ```
    or for VS2022+
    ```
    cmake -G "Visual Studio 17 2022" ..
    ```
1. Compile the solution for release
    ```
    msbuild ALL_BUILD.vcxproj /p:Configuration=Release
    ```

This SDK is known to work with these CMake generators:
* Visual Studio 17 2022 Win64
* Visual Studio 16 2019 Win64
* Visual Studio 15 2017 Win64
* Visual Studio 14 2015 Win64
* Visual Studio 12 2013 Win64
* Visual Studio 11 2012 Win64
* Unix MakeFiles

After running 'msbuild' you can edit,test your server sdk with solutions file 'out/gamelift-server-sdk/aws-cpp-sdk-gamelift-server.sln'

### CMake options
#### BUILD_FOR_UNREAL

Optional variable to build the SDK for use with Unreal Engine. Set variable to 1 to create an Unreal build. Default setting is false.
With this option enabled, all dependencies are built statically and then rolled into a single shared object library.
```sh
cmake -DBUILD_FOR_UNREAL=1 ..
```

#### BUILD_SHARED_LIBS

Optional variable to select either a static or dynamic build. Default setting is **static**. Set variable to 1 for dynamic.

```sh
cmake -DBUILD_SHARED_LIBS=1 ..
```

#### GAMELIFT_USE_STD

Optional variable to choose whether or not to use the C++ STD when building. Default setting is **true**. 

The Amazon GameLift server SDK can be built to use the C++ standard library (#include <cstdio> and using namespace std;), otherwise C library functionality (#include <string.h> for example) is used instead.

To build the server SDK libraries using the C++ standard library, use the usual form for building the SDK. E.g. for Windows you might use:

```sh
mkdir out
cd out
cmake -G "Visual Studio 14 2015 Win64" ..
msbuild ALL_BUILD.vcxproj /p:Configuration=Release
```
This implies the use of the default -DGAMELIFT_USE_STD=1 flag. To turn off the dependency with the C++ std:: library, the following cmake command should be substituted.

    cmake -G "Visual Studio 14 2015 Win64" -DGAMELIFT_USE_STD=0 ..

Whether or not the C++ standard library should be used is generally a matter of preference, but there are some considerations.

* Use of -DBUILD_FOR_UNREAL=1 will override use of -DGAMELIFT_USE_STD=1 and the std:: library will not be used in an unreal build.
* Use of the -DGAMELIFT_USE_STD=0 flag modifies certain SDK API function prototypes, obviously because std::string will not be accepted as a parameter in this case. It is important when you are using libraries build with -DGAMELIFT_USE_STD=1 in your application that you continue to define the GAMELIFT_USE_STD=1 preprocessor definition prior to including the headers:
  ```sh
    #define GAMELIFT_USE_STD=1  
    #include "aws\gamelift\server\GameLiftServerAPI.h"
  ```
  Otherwise the prototype in the built library will not match the prototype in the header that you are including and you will get compiler errors such as these:
    ```sh
  error C2039: 'InitSDKOutcome' : is not a member of 'Aws::GameLift::Server'
    error C2065: 'InitSDKOutcome' : undeclared identifier
    ```
  and others.  
  Likewise, if the libraries are built with -DGAMELIFT_USE_STD=0 then the following is usual (though not required):
  ```sh
    #define GAMELIFT_USE_STD=0
    #include "aws\gamelift\server\GameLiftServerAPI.h" GAMELIFT_USE_STD 
  ```

#### CMAKE_BUILD_TYPE

Option to choose whether to build in debug or release.  Options are Debug or Release (case sensitive).  Default is Release.
```sh
cmake -DCMAKE_BUILD_TYPE=Debug ..
```

#### RUN_CLANG_FORMAT

Option to automatically run clang-format as part of the build process over all SDK and test source code. This will take some time, but correct any styling inconsistencies.

## Common Issues
#### File path too long errors when running msbuild
* **Cause**: There is a long running issue with Visual Studio where regardless of settings for long file/folder paths in Windows Registry, some of the file IO functions will still only allow for a short file/folder path.
* **Mitigation**: Will not ideal, the mitigation for this issue is to make sure your are building the Amazon GameLift and AWS SDKs with as short of a folder structure as possible. If you encounter the issue, then try moving your project to a folder closer to the root.

#### Runtime error for missing libssl and/or libcrypto DLLs
```
LogWindows: Failed to load ‘../GameLiftServerSDK/ThirdParty/GameLiftServerSDK/Win64/aws-cpp-sdk-gamelift-server.dll’ (GetLastError=0)
LogWindows: Missing import: libssl-3-x64.dll
LogWindows: Missing import: libcrypto-3-x64.dll
```
* **Cause**: When uploading your application to Amazon GameLift, the related OpenSSL DLLs were not included in the applications path.
* **Mitigation**: Using the same version of OpenSSL that you built the Amazon GameLift SDK with, go to the OpenSSL folder and find the libssl-3-x64.dll and libcrypto-3-x64.dll DLLs in the root folder. Reupload your application with these DLLs included.
